#!/usr/bin/perl
#------------------------------------------------------------------------------
# Abstract:
#    Simple program to recover forgotten WS_FTP passwords.
#
# Design:
#   WS_FTP keep site passwords in a file named ws_ftp.ini in the main WS_FTP
#   directory.  Early versions (up to and including ver. 5.06 - 951112)
#   "encrypted" the passwords using a very simple algorithm (for this
#   algorithm, see the subroutine "recover").
#
# Usage:
#   wsftpcrc.pl [file]
#
# Authorship:
#    Copyright (c) 2003-2006 Gisle Hannemyr.
#    Permission is granted to distribute copies of this program as long as
#    this notice and the other copyright notices are not removed.
#    If you intend to distribute changed versions of this program, please make
#    an entry in the "history" log (below) and mark the changed/added lines
#    with your initials. I maintain the program, and shall appreciate copies
#    of bug fixes and extensions.
#    Bug reports, comments and improvements to:
#       email: gisle@hannemyr.no
#    Website:
#       http://hannemyr.com/enjoy/wsftpcrk.html
#
# History:
#    [gh] = Gisle Hannemyr
#    1.01 2006 Oct 15 [gh] Check for non-recoverable pw.
#    1.00 2003 Mar 18 [gh] Wrote it.
#
# Portability:
#    Belived to be portable (but only tested under Solaris 5.8).
#    Tested on WS_FTP32 (LE), version (5.06) 951112
#
# Bugs:
#    None known.
#------------------------------------------------------------------------------


#---( subrutines )-------------------------------------------------------------

# Abs: Recover hex encoded password.
# Par: $crypt = the "encrypted" password.
# Ret: The recovered password.
sub recover0 {
    local($crypt) = pop(@_);

    $retrn = "";
    $aa = 0;
    while ($crypt) {
	$char2 = substr($crypt,0,2);
	$crypt = substr($crypt,2);
        $ascii = hex($char2) - $aa;
	$retrn = $retrn . chr($ascii);
	$aa++;
    }
    return $retrn;;
} # recover0


#---( main )-------------------------------------------------------------------

print STDERR "wsftpcrk.pl - Recover WS_FTP passwords. ver. 1.01\n\n";


while ($ARGV[0] =~ /^-/) {
    $_ = shift;
    if (/^-h(.*)$/) {
	$ishelp++;
    } else {
	$boguso++;
    }
}

if ($boguso) {
    print "Error: Urecognized option passed to script.\n\n";
}

if ($ishelp || $boguso) {
    print <<"EOF";
Usage:
    wsftpcrk.pl [file]

Simple program to recover forgotten WS_FTP-passwords.  If you don\'t
specify a file name, it will look for  \"ws_ftp.ini\"  in the current
directory.  WS_FTP keep site passwords in a file named \"ws_ftp.ini\"
in the main WS_FTP directory  -  and you should copy this file to a 
working directory before running wsftpcrk.pl.

For updates, look at the website:
    http://hannemyr.com/enjoy/wsftpcrk.html

EOF

    exit 1;
}

if (0 <= $#ARGV) {
    $IniFil = shift;
} else {
    $IniFil = 'ws_ftp.ini';
}

# Read in the password++ file
open(FILH, $IniFil) || die "Can't open \"$IniFil\".\n$!.\n";

$noofrecs = -1;
while (<FILH>) {
    chop;
    chop;
    if (substr($_, 0, 1) eq '[') {
	$noofrecs++;
	chop;
        $id = substr($_, 1);
	$Sites[$noofrecs]->{name} = $id;
	next;
    }
    if (substr($_, 0, 4) eq 'HOST') {
        $id = substr($_, 5);
	$Sites[$noofrecs]->{host} = $id;
	next;
    }
    if (substr($_, 0, 3) eq 'PWD') {
        $id = substr($_, 4);
	# Only know how to recover hex encoded
	$nn = hex(substr($_, 4, 8));
	$Sites[$noofrecs]->{crpt} = $id;
	$Sites[$noofrecs]->{pass} = ($nn) ? recover0($id) : "<not recovered>";
	next;
    }
} # while
close(FILH);
$noofrecs++;

# Generate report
print "========================================================================\n";
print "Total sites in \"$IniFil\": $noofrecs\n";
print "------------------------------------------------------------------------\n";
for ($ii=0; $ii < $noofrecs; $ii++) {
    if ($Sites[$ii]{crpt}) {
	print "{ $ii: ";
	print "$Sites[$ii]{name}";
	print "; $Sites[$ii]{host}";
       #print "; $Sites[$ii]{crpt}";
        print "; $Sites[$ii]{pass} ";
	print "}\n";
    }
}
print "========================================================================\n";

# EOF
